package com.elasticjee.api.enterprise;

/**
 * @author yangck
 */
public enum EnterpriseStatus {
    /**
     * {@code 310 未启用}
     */
    UNAPPLY(310, "未启用"),
    /**
     * {@code 311 申请中}
     */
    UNAUDIT(311, "申请中"),
    /**
     * {@code 312 未激活}
     */
    UNACT(312, "未激活"),
    /**
     * {@code 313 已激活}
     */
    ENABLED(313, "已激活"),
    /**
     * {@code 314 已禁用}
     */
    DISABLED(314, "已禁用"),
    /**
     * {@code 315 已注销,已取消}
     */
    CANCELLED(315, "已注销");

    EnterpriseStatus(int value, String phrase) {
        this.value = value;
        this.phrase = phrase;
    }

    private final int value;

    private final String phrase;

    /**
     * @return 状态的整型编码
     */
    public int value() {
        return this.value;
    }

    /**
     * @return 状态的描述
     */
    public String getPhrase() {
        return this.phrase;
    }

    /**
     * @param statusCode
     *            状态的编码
     * @return 状态
     * @throws IllegalArgumentException
     *             如果statusCode不存在的话
     */
    public static EnterpriseStatus valueOf(int statusCode) {
        for (EnterpriseStatus status : values()) {
            if (status.value == statusCode) {
                return status;
            }
        }
        throw new IllegalArgumentException("没有与编号 [" + statusCode + "]匹配的状态");
    }

    /**
     * 返回状态的编号
     */
    @Override
    public String toString() {
        return Integer.toString(value);
    }

}
