package com.elasticjee.user;

import com.alibaba.fastjson.annotation.JSONField;
import com.elasticjee.api.enterprise.Enterprise;
import org.codehaus.jackson.annotate.JsonIgnore;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.NotFound;
import org.hibernate.annotations.NotFoundAction;
import org.springframework.util.CollectionUtils;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

/**
 * @author yangck
 */
@Entity
@Table(name = "sec$users")
//@Cacheable
//@Cache(usage = CacheConcurrencyStrategy.READ_WRITE, region = "com.uas.platform.b2b.model.User")
public class User implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "users_gen")
    @SequenceGenerator(name = "users_gen", sequenceName = "users_seq", allocationSize = 1)
    @Column(name = "user_uu")
    private Long userUU;

    @Column(name = "user_name", length = 50)
    private String userName;

    @Column(name = "user_pwd", length = 32)
    private String userPwd;

    @Column(name = "user_email", length = 50, nullable = false)
    @NotNull
    private String userEmail;

    @Column(name = "user_tel", length = 30)
    private String userTel;

    @Column(name = "user_sex", length = 10)
    private String userSex;

    @Column(name = "user_idcode", length = 20)
    private String userIdcode;

    private Short enable;

    private Short issys;

    @Column(name = "user_imid")
    private Long userIMId;

    /**
     * 用户类型
     */
    @Column(name = "user_type", length = 20)
    private String userType;

    /**
     * 所属公司
     */
    @ManyToMany(cascade = { CascadeType.REFRESH, CascadeType.REMOVE }, fetch = FetchType.EAGER)
    @JoinTable(name = "sec$userenterprise", joinColumns = @JoinColumn(name = "user_uu", referencedColumnName = "user_uu"), inverseJoinColumns = @JoinColumn(name = "en_uu", referencedColumnName = "en_uu"))
    @OrderBy("uu")
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    private Set<Enterprise> enterprises;

    /**
     * 所属公司
     */
    @Transient
    @NotFound(action = NotFoundAction.IGNORE)
    private Enterprise enterprise;

    /**
     * 最近登录IP地址
     */
    @Transient
    @NotFound(action = NotFoundAction.IGNORE)
    private String ip;

    @Column(name = "user_imageurl")
    private String imageUrl;

    public Long getUserIMId() {
        return userIMId;
    }

    public void setUserIMId(Long userIMId) {
        this.userIMId = userIMId;
    }

    public Long getUserUU() {
        return userUU;
    }

    public void setUserUU(Long userUU) {
        this.userUU = userUU;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public String getUserPwd() {
        return userPwd;
    }

    public void setUserPwd(String userPwd) {
        this.userPwd = userPwd;
    }

    public String getUserEmail() {
        return userEmail;
    }

    public void setUserEmail(String userEmail) {
        this.userEmail = userEmail;
    }

    public String getUserTel() {
        return userTel;
    }

    public void setUserTel(String userTel) {
        this.userTel = userTel;
    }

    public String getUserSex() {
        return userSex;
    }

    public void setUserSex(String userSex) {
        this.userSex = userSex;
    }

    public String getUserIdcode() {
        return userIdcode;
    }

    public void setUserIdcode(String userIdcode) {
        this.userIdcode = userIdcode;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public Short getEnable() {
        return enable;
    }

    public void setEnable(Short enable) {
        this.enable = enable;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public Short getIssys() {
        return issys;
    }

    public void setIssys(Short issys) {
        this.issys = issys;
    }

    public String getUserType() {
        return userType;
    }

    public void setUserType(String userType) {
        this.userType = userType;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public Set<Enterprise> getEnterprises() {
        return enterprises;
    }

    public void setEnterprises(Set<Enterprise> enterprises) {
        this.enterprises = enterprises;
    }

    public Enterprise getEnterprise() {
        return enterprise;
    }

    public void setEnterprise(Enterprise enterprise) {
        this.enterprise = enterprise;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public String getIp() {
        return ip;
    }

    public void setIp(String ip) {
        this.ip = ip;
    }

    public String getImageUrl() {
        return imageUrl;
    }

    public void setImageUrl(String imageUrl) {
        this.imageUrl = imageUrl;
    }



    /**
     * 获取用户所属企业UU号
     *
     * @return
     */
    @JSONField(serialize = false)
    @JsonIgnore
    public Set<Long> getEnUUs() {
        Set<Long> enUUs = new HashSet<>();
        if (!CollectionUtils.isEmpty(enterprises)) {
            for (Enterprise en : enterprises) {
                enUUs.add(en.getUu());
            }
        }
        return enUUs;
    }

    public void addEnterprise(Enterprise enterprise) {
        if (this.enterprises == null) {
            Set<Enterprise> enterprises = new HashSet<Enterprise>();
            enterprises.add(enterprise);
            this.enterprises = enterprises;
            return;
        }
        if (!this.enterprises.contains(enterprise)) {
            this.enterprises.add(enterprise);
        }
        if (this.enterprise == null) {
            this.enterprise = enterprise;
        }
    }

    public void removeEnterprise(Enterprise enterprise) {
        if (this.enterprises.contains(enterprise)) {
            this.enterprises.remove(enterprise);
        }
    }

    public void setCurrentEnterprise() {
        if (this.enterprises != null && this.enterprises.size() > 0)
            this.enterprise = this.enterprises.iterator().next();
    }

    public void setCurrentEnterprise(long enUU) {
        if (this.enterprises != null && this.enterprises.size() > 0)
            for (Enterprise enterprise : enterprises) {
                if (enterprise.getUu() == enUU) {
                    this.enterprise = enterprise;
                    break;
                }
            }
        if (this.enterprise == null)
            setCurrentEnterprise();
    }
}
