package com.elasticjee.user;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import java.security.Principal;
import java.util.Optional;

/**
 * @author yangck
 */
@RestController
public class UserController {

    @Autowired
    private UserService userService;

    @RequestMapping("/user")
    public Principal user(Principal user) {
        return user;
    }

    @RequestMapping(value = "/users", method = RequestMethod.POST, name = "createUser")
    public ResponseEntity<User> createUser(@RequestBody User user) {
        Assert.notNull(user);
        return Optional.ofNullable(userService.createUser(user))
                .map(result -> new ResponseEntity<>(result, HttpStatus.CREATED))
                .orElse(new ResponseEntity<>(HttpStatus.CONFLICT));
    }

    @RequestMapping(value = "users/{id}", method = RequestMethod.GET, name = "getUser")
    public ResponseEntity<User> getUser(@PathVariable("id") Long userUu) {
        return Optional.ofNullable(userService.getUserByUserUU(userUu))
                .map(result -> new ResponseEntity<>(result, HttpStatus.OK))
                .orElse(new ResponseEntity<>(HttpStatus.NOT_FOUND));
    }

    @RequestMapping(value = "users/{userUu}", method = RequestMethod.PUT, name = "updateUser")
    public ResponseEntity<User> updateUser(@PathVariable(value = "userUu") Long userUu, @RequestBody User user) {
        Assert.notNull(user);
        user.setUserUU(userUu);
        return Optional.ofNullable(userService.updateUser(userUu, user))
                .map(result -> new ResponseEntity<>(result, HttpStatus.OK))
                .orElse(new ResponseEntity<>(HttpStatus.NOT_FOUND));
    }

    @RequestMapping(value = "users/{userUu}", method = RequestMethod.DELETE,
            name = "deleteUser")
    public ResponseEntity<Boolean> deleteUser(@PathVariable("userUu") Long userUu) {
        return Optional.ofNullable(userService.deleteUser(userUu))
                .map(result -> new ResponseEntity<>(result, HttpStatus.NO_CONTENT))
                .orElse(new ResponseEntity<>(HttpStatus.NOT_FOUND));
    }
}
