package com.elasticjee.user;

import com.elasticjee.util.Regex;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

/**
 * @author yangck
 */
@Service
public class UserService {

    @Autowired
    private UserRepository userRepository;

    @CacheEvict(value = "user", key = "#user.getUserUU()")
    public User createUser(User user) {
        User result = null;
        if(!userRepository.exists(user.getUserUU())) {
            result = userRepository.save(user);
        }
        return result;
    }

    @Cacheable(value = "user", key = "#userUU")
    public User getUserByUserUU(Long userUU) {
        return userRepository.findOne(userUU);
    }

    @CachePut(value = "user", key = "#userUU")
    public User updateUser(Long userUU, User user) {
        User result = null;
        if(userRepository.exists(user.getUserUU())) {
            result = userRepository.save(user);
        }
        return result;
    }

    @CacheEvict(value = "user", key = "#userUU")
    public boolean deleteUser(Long userUU) {
        boolean deleted = false;
        if (userRepository.exists(userUU)) {
            userRepository.delete(userUU);
            deleted = true;
        }
        return deleted;
    }

    /**
     * 支持三个唯一账号：邮箱、手机号、ID
     *
     * @param username
     * @return
     */
    public User getUserByUserName(String username) {
        if (username.contains("@")) {
            return userRepository.findUserByUserEmail(username);
        } else if (username.matches(Regex.REGEX_MOBILE)) {
            return userRepository.findUserByUserTel(username);
        } else {
            return userRepository.findUserByUserUU(username);
        }
    }


}
